typedef double        LDBL;
typedef unsigned char UCHAR;
typedef unsigned      Word;
typedef unsigned char Byte;

#define dMIN2(x,y)   (((x)<(y))?(x):(y))
#define dMAX2(x,y)   (((x)>(y))?(x):(y))
#define dMAX3(x,y,z) (((dMAX2(x,y))>(z))?(dMAX2(x,y)):(z))
#define Round(x)     (((x)>0)?(int)(x+0.5):(int)(x-0.5))
#define dABS(x)      (((x)>0)?(x):(-x))

#define pi 3.14159265354
#define Si 1
#define No 0

//	Tipo de objeto

#define INDEFINIDO    (UCHAR)(0)
#define ESFERA        (UCHAR)(1)
#define PLANO         (UCHAR)(2)
#define CUADRICA      (UCHAR)(3)
#define ELIPSOIDE     (UCHAR)(4)

//	Tipo de rayo

#define SOMBRA     (UCHAR)(0)
#define PRIMARIO   (UCHAR)(1)
#define REFLEJADO  (UCHAR)(2)
#define REFRACTADO (UCHAR)(3)

//	Tipo de recorte de color

#define CLAMP (UCHAR)(0)
#define SCALE (UCHAR)(1)
#define CLIP  (UCHAR)(2)
#define COTA  (UCHAR)(3)

// Tipo de matriz

#define M_CAMBIO_BASE (UCHAR)(0)

/************************************************************************
************************************************************************/

typedef struct vector3DEsf
	{
	LDBL r;
	LDBL theta;
	LDBL phi;
	} vector3DEsf;

typedef struct vector3D
	{
	LDBL x;
	LDBL y;
	LDBL z;
	LDBL t;
	} vector3D;

typedef struct vector2D
	{
	LDBL x;
	LDBL y;
	} vector2D;

typedef struct colorRGB
	{
	LDBL r;
	LDBL g;
	LDBL b;
	} colorRGB;

typedef struct matriz4x4
	{
	UCHAR     tipo;
	vector3D *f1;
	vector3D *f2;
	vector3D *f3;
	vector3D *f4;
	} matriz4x4;

typedef struct rayo
	{
	UCHAR    tipo;
	LDBL		iR;			// Indice de refraccin
	LDBL		d;				// Distancia recorrida por el rayo
	vector3D *origen;
	vector3D *dir;
	colorRGB *color;
	} rayo;

typedef struct volumenVision
	{
	vector3D *ojo;
	LDBL   	 angVisVertical;
	LDBL	  	 angVisHorizontal;
	} volumenVision;

typedef struct object
	{
	long				   posicion;
	struct matriz4x4 *ecuacion;
/*
	Ax^2+By^2+Cz^2+Dxy+Exz+Fyz+Gx+Hy+Iz+J=0

	fila1->x = A
	fila1->y = D/2
	fila1->z = E/2
	fila1->t = G/2
	fila2->x = D/2
	fila2->y = B
	fila2->z = F/2
	fila2->t = H/2
	fila3->x = E/2
	fila3->y = F/2
	fila3->z = C
	fila3->t = I/2
	fila4->x = G/2
	fila4->y = H/2
	fila4->z = I/2
	fila4->t = J
*/
	struct colorRGB  *color_difuso;
	struct colorRGB  *color_especular;
	LDBL		  Ka;						/* Coef. reflexion ambiental */
	LDBL		  Kd;						/* Coef. reflexion difusa */
	LDBL		  Kh;						/* Coef. 'highlight' (especular) */
	struct vector3D  *Ks;			/* Coef. reflexion especular */
	LDBL		   n;						/* Exponente de reflexion especular */
	struct vector3D  *Kt;			/* Coef. de transmisin */
	LDBL		  iR;						/* Indice de refraccion */
	struct object    *objetoPrevio;
	struct object    *objetoSiguiente;
	} object;

typedef struct light
	{
	long		 	  posicion;
	vector3D 	 *origen;
	vector3D 	 *intensidad;
	LDBL			  c0;			// Coefientes de atenuacion de la luz:
	LDBL			  c1;			// f_att = min( 1/( c1 + c2*d + c3*d^2 ), 1)
	LDBL			  c2;
	struct light *focoPrevio;
	struct light *focoSiguiente;
	} light;

typedef struct interseccion
	{
	vector3D *pto;
	vector3D *normal;
	object   *objeto;
	} interseccion;


/************************************************************************/

typedef struct listaObjects
	{
	long	       	nObjetos;
	struct object *objetoInicial;
	struct object *objetoFinal;
	} listaObjects;

typedef struct listaLights
	{
	long	       	nFocos;
	struct light  *focoInicial;
	struct light  *focoFinal;
	} listaLights;